using System;
using System.IO;
using System.ComponentModel;
using System.Text;

namespace id.Quake.Files {

	[SupportedExtension("lmp")]
	public class ColourMap : IPicture, INamedResource {

		#region Properties


		private byte[][] map;
		public byte[][] Map {
			get { return this.map; }
		}

		private string name;
		[Browsable(false)]
		public string Name { get { return this.name; } set { this.name = value; } }

		[Browsable(false)]
		public bool NamedInternally { get { return false; } }

		private readonly ResourceLoader Resources;

		#endregion


		#region Constructor

		public ColourMap(ResourceLoader resources, Stream stream) {
			this.Resources = resources;
			BinaryReader ColourMapReader = new BinaryReader(stream);
			this.map = new byte[256][];
			for (int i = 0; i < 256; ++i) this.map[i] = new byte[64];
			for (int b = 0; b < 64; ++b) {
				for (int c = 0; c < 256; ++c) {
					this.map[c][b] = ColourMapReader.ReadByte();
				}
			}
		}

		#endregion


		#region IPicture Members

		public Picture Picture {
			get {
				int Width = 256;
				int Height = 64;
				int[] Data = new int[Width * Height];

				Palette P;
				if (!this.Resources.TryGetCachedObject("gfx/palette.lmp", out P)) {
					P = new Palette(this.Resources, this.Resources.OpenFile("gfx/palette.lmp"));
					this.Resources.CacheObject("gfx/palette.lmp", P);
				}

				for (int b = 0, d = 0; b < 64; ++b) {
					for (int c = 0; c < 256; ++c) {
						Data[d++] = P.Colours[this.map[c][b]];
					}
				}

				return new Picture(Width, Height, Data);
			}
		}

		#endregion
	}
}
