using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace id.Quake.Files {

	public partial class Level {

		/// <summary>A directory entry within the level.</summary>
		public class Entry {


			/// <summary>Used to index the various entries.</summary>
			public enum EntryTypes {
				Entities,
				Planes,
				MipTextures, // Q1
				Vertices,
				VisibilityLists,
				Nodes,
				TextureInfo,
				Faces,
				LightMaps,
				ClipNodes,
				Leaves,
				FaceList,
				Edges,
				EdgeList,
				Models,
				LeafBrushes, // Q2
				Brushes, // Q2
				BrushSides, // Q2
				Pop, // Q2
				Areas, // Q2
				AreaPortals, // Q2
				LeafBrushTable, // Q2
			}

			private readonly uint offset;
			/// <summary>Gets the offset of the data within the level in bytes.</summary>
			public uint Offset {
				get { return this.offset; }
			}
			private readonly uint length;
			/// <summary>Gets the size of the data in bytes.</summary>
			public uint Length {
				get { return this.length; }
			}

			private readonly Level level;
			/// <summary>Gets the level containing this entry.</summary>
			public Level Level {
				get { return this.level; }
			}

			/// <summary>Create an instance of an entry from a level file.</summary>
			/// <param name="level">The level file to load an entry from.</param>
			public Entry(Level level) {
				this.level = level;
				BinaryReader EntryReader = new BinaryReader(level.BaseStream);
				this.offset = EntryReader.ReadUInt32();
				this.length = EntryReader.ReadUInt32();
			}

			/// <summary>Gets a stream for reading from.</summary>
			/// <returns>A stream to load data from.</returns>
			/// <remarks>Do not modify the contents of the returned stream.</remarks>
			public Stream GetStream() {
				this.Level.BaseStream.Seek(this.Offset, SeekOrigin.Begin);
				return this.Level.BaseStream;
			}

			/// <summary>Gets a reader for reading binary data from.</summary>
			/// <returns>A BinaryReader to read data from.</returns>
			public BinaryReader GetBinaryReader() {
				return new BinaryReader(this.GetStream());
			}

		}

	}
}