using System;
using System.IO;
using System.ComponentModel;
using System.Text;

namespace id.Quake.Files {

	[SupportedExtension("wal")]
	public class WallTexture : IPicture, IMipMappedPicture, INamedResource {

		#region Properties

		private readonly string name;
		/// <summary>Gets the name of the texture.</summary>
		[Description("The internal filename of the texture.")]
		[Category("Name")]
		public string Name {
			get { return this.name; }
			set { throw new InvalidOperationException("The name of this resource cannot be changed."); }
		}

		[Browsable(false)]
		public bool NamedInternally { get { return true; } }

		private readonly string nextName;
		/// <summary>Name of the next texture in the animation sequence.</summary>
		[Description("Name of the next texture in the animation sequence if applicable.")]
		[Category("Name")]
		public string NextName {
			get { return this.nextName; }
		}

		private readonly int width;
		/// <summary>Width, in pixels, of the texture.</summary>
		[Description("Width of the texture in pixels.")]
		public int Width {
			get { return this.width; }
		}

		private readonly int height;
		/// <summary>Height, in pixels, of the texture.</summary>
		[Description("Height of the texture in pixels.")]
		public int Height {
			get { return this.height; }
		}

		private int[] offsets;
		private uint flags;
		private uint contents;
		private uint value;

		private byte[] Data;

		private readonly ResourceLoader Resources;

		#endregion

		#region Methods


		#endregion

		#region Constructor

		public WallTexture(ResourceLoader resources, Stream stream) {

			this.Resources = resources;
			
			BinaryReader WalReader = new BinaryReader(stream);
			this.name = Encoding.ASCII.GetString(WalReader.ReadBytes(32)).Split('\0')[0];
			this.width = WalReader.ReadInt32();
			this.height = WalReader.ReadInt32();

			this.offsets = new int[4];
			for (int i = 0; i < 4; ++i) this.offsets[i] = WalReader.ReadInt32();

			this.nextName = Encoding.ASCII.GetString(WalReader.ReadBytes(32)).Split('\0')[0];

			this.flags = WalReader.ReadUInt32();
			this.contents = WalReader.ReadUInt32();
			this.value = WalReader.ReadUInt32();

			stream.Seek(0, SeekOrigin.Begin);

			this.Data = WalReader.ReadBytes((int)stream.Length);

		}

		#endregion

		#region IPicture Members

		public Picture Picture {
			get {
				return this.GetPicture(0);
			}
		}

		public Picture GetPicture(int mipLevel) {
			

			int Width = this.Width >> mipLevel;
			int Height = this.Height >> mipLevel;

			ZSoftPcx P;
			if (!this.Resources.TryGetCachedObject("pics/colormap.pcx", out P)) {
				P = this.Resources.LoadObject<ZSoftPcx>("pics/colormap.pcx");
				this.Resources.CacheObject("pics/colormap.pcx", P);
			}

			byte[] DataToRead = new byte[Width * Height];
			Array.Copy(this.Data, this.offsets[mipLevel], DataToRead, 0, DataToRead.Length);
			return new Picture(Width, Height, Array.ConvertAll<byte, int>(DataToRead, delegate(byte b) { return (P.Palette[b] & 0xFFFFFF) | (b << 24); }));
		}

		#endregion

		


	}
}
