using System;
using System.Collections.Generic;

namespace id.Quake.Files {
	public partial class QuakeC {

		private int ArgumentCount;

		private void Execute(Function function) {

			int EntryDepth = this.FunctionStack.Count;
			this.EnterFunction(function);

			for (; ; ) {

				// Fetch a statement.
				Statement CurrentStatement = this.Statements[this.StatementPointer++];

				VariablePointer A = CurrentStatement.OperandA;
				VariablePointer B = CurrentStatement.OperandB;
				VariablePointer C = CurrentStatement.OperandC;
				switch (CurrentStatement.OperationCode) {
					case Statement.Operation.OP_ADD_F:
						C.Float = A.Float + B.Float;
						break;
					case Statement.Operation.OP_ADD_V:
						C.Vector = A.Vector + B.Vector;
						break;
					case Statement.Operation.OP_SUB_F:
						C.Float = A.Float - B.Float;
						break;
					case Statement.Operation.OP_SUB_V:
						C.Vector = A.Vector - B.Vector;
						break;
					case Statement.Operation.OP_MUL_F:
						C.Float = A.Float * B.Float;
						break;
					case Statement.Operation.OP_MUL_V: {
							Vector3 a = A.Vector; Vector3 b = B.Vector;
							C.Float = a.X * b.X + a.Y * b.Y + a.Z * b.Z;
						}
						break;
					case Statement.Operation.OP_MUL_FV:
						C.Vector = A.Float * B.Vector;
						break;
					case Statement.Operation.OP_MUL_VF:
						C.Vector = B.Float * A.Vector;
						break;
					case Statement.Operation.OP_DIV_F:
						C.Float = A.Float / B.Float;
						break;
					case Statement.Operation.OP_BITAND:
						C.Float = (int)A.Float & (int)B.Float;
						break;
					case Statement.Operation.OP_BITOR:
						C.Float = (int)A.Float | (int)B.Float;
						break;
					case Statement.Operation.OP_GE:
						C.Float = (A.Float >= B.Float) ? 1f : 0f;
						break;
					case Statement.Operation.OP_LE:
						C.Float = (A.Float <= B.Float) ? 1f : 0f;
						break;
					case Statement.Operation.OP_GT:
						C.Float = (A.Float > B.Float) ? 1f : 0f;
						break;
					case Statement.Operation.OP_LT:
						C.Float = (A.Float < B.Float) ? 1f : 0f;
						break;
					case Statement.Operation.OP_AND:
						C.Float = (A.Float != 0 && B.Float != 0) ? 1f : 0f;
						break;
					case Statement.Operation.OP_OR:
						C.Float = (A.Float != 0 || B.Float != 0) ? 1f : 0f;
						break;

					case Statement.Operation.OP_NOT_F:
						C.Boolean = !A.Boolean;
						break;
					case Statement.Operation.OP_NOT_V:
						C.Boolean = (A.Vector == Vector3.Zero);
						break;
					case Statement.Operation.OP_NOT_S:
						C.Boolean = A.Integer != 0 || !string.IsNullOrEmpty(A.String);
						break;
					case Statement.Operation.OP_NOT_FNC:
						C.Boolean = A.Integer != 0;
						break;
					//TODO: OP_NOT_ENT
					case Statement.Operation.OP_EQ_F:
						C.Boolean = A.Float == B.Float;
						break;
					case Statement.Operation.OP_EQ_V:
						C.Boolean = A.Vector == B.Vector;
						break;
					case Statement.Operation.OP_EQ_S:
						C.Boolean = A.String == B.String;
						break;
					case Statement.Operation.OP_EQ_E:
					case Statement.Operation.OP_EQ_FNC:
						C.Boolean = A.Integer == B.Integer;
						break;

					case Statement.Operation.OP_NE_F:
						C.Boolean = A.Float != B.Float;
						break;
					case Statement.Operation.OP_NE_V:
						C.Boolean = A.Vector != B.Vector;
						break;
					case Statement.Operation.OP_NE_S:
						C.Float = string.Compare(A.String, B.String);
						break;
					case Statement.Operation.OP_NE_E:
					case Statement.Operation.OP_NE_FNC:
						C.Boolean = A.Integer != B.Integer;
						break;

					case Statement.Operation.OP_STORE_F:
					case Statement.Operation.OP_STORE_S:
					case Statement.Operation.OP_STORE_ENT:
					case Statement.Operation.OP_STORE_FLD:
					case Statement.Operation.OP_STORE_FNC:
						B.Integer = A.Integer;
						break;
					case Statement.Operation.OP_STORE_V:
						B.Vector = A.Vector;
						break;

					case Statement.Operation.OP_IFNOT:
						if (!A.Boolean) StatementPointer += (short)CurrentStatement.OperandIndices[1] - 1;
						break;
					case Statement.Operation.OP_IF:
						if (A.Boolean) StatementPointer += (short)CurrentStatement.OperandIndices[1] - 1;
						break;
					case Statement.Operation.OP_GOTO:
						StatementPointer += (short)CurrentStatement.OperandIndices[0] - 1;
						break;


					case Statement.Operation.OP_STOREP_F:
					case Statement.Operation.OP_STOREP_ENT:
					case Statement.Operation.OP_STOREP_FLD:
					case Statement.Operation.OP_STOREP_S:
					case Statement.Operation.OP_STOREP_FNC:
						this.Edicts.GetField(B.Integer).Integer = A.Integer;
						break;
					case Statement.Operation.OP_STOREP_V:
						this.Edicts.GetField(B.Integer).Vector = A.Vector;
						break;

					case Statement.Operation.OP_ADDRESS:
						C.Integer = A.Edict.DataOffset + B.Integer;
						break;

					case Statement.Operation.OP_LOAD_F:
					case Statement.Operation.OP_LOAD_FLD:
					case Statement.Operation.OP_LOAD_ENT:
					case Statement.Operation.OP_LOAD_S:
					case Statement.Operation.OP_LOAD_FNC:
						C.Integer = this.Edicts.GetField(A.Edict.DataOffset + B.Integer).Integer;
						break;
					case Statement.Operation.OP_LOAD_V:
						C.Vector = this.Edicts.GetField(A.Edict.DataOffset + B.Integer).Vector;
						break;
					


					case Statement.Operation.OP_CALL0:
					case Statement.Operation.OP_CALL1:
					case Statement.Operation.OP_CALL2:
					case Statement.Operation.OP_CALL3:
					case Statement.Operation.OP_CALL4:
					case Statement.Operation.OP_CALL5:
					case Statement.Operation.OP_CALL6:
					case Statement.Operation.OP_CALL7:
					case Statement.Operation.OP_CALL8:

						this.ArgumentCount = (int)CurrentStatement.OperationCode - (int)Statement.Operation.OP_CALL0;

						QuakeC.Function Func = A.Function;
						if (Func.FirstStatement < 0) {
							this.BuiltInFunctions[-Func.FirstStatement]();
						} else {
							this.Execute(Func);
						}
						break;

					case Statement.Operation.OP_DONE:
					case Statement.Operation.OP_RETURN:
						this.SetGlobal(GlobalVariableOffsets.Return, A.Vector);
						this.LeaveFunction();
						if (this.FunctionStack.Count == EntryDepth) return;
						break;

					case Statement.Operation.OP_STATE:
						//TODO: Check #ifdef FPS_20
						this.Self.Properties["nextthink"].Float = this.Globals["time"].Value.Float + 0.1f;
						this.Self.Properties["frame"].Float = A.Float;
						this.Self.Properties["think"].Integer = B.Integer; // Assign function.
						break;

					default:
						throw new InvalidOperationException("Operation " + CurrentStatement.OperationCode.ToString() + " not supported.");
				}
			}
		}
	
	}
}
