using System;
using System.ComponentModel;

namespace id.Quake {

	/// <summary>Defines a simple bounding box made up of short vectors.</summary>
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public struct BoundingBoxShort {

		/// <summary>Gets or sets the minimum position in the bounding box.</summary>
		public Vector3Short Min;
		/// <summary>Gets or sets the maximum position in the bounding box.</summary>
		public Vector3Short Max;

		/// <summary>Creates an instance of the bounding box from two positions.</summary>
		/// <param name="min">The smallest possible position in the bounding box.</param>
		/// <param name="max">The largest possible position in the bounding box.</param>
		public BoundingBoxShort(Vector3Short min, Vector3Short max) {
			this.Min = min;
			this.Max = max;
		}

		/// <summary>Check if a bounding box completely contains another.</summary>
		public bool CompletelyContains(BoundingBoxShort b) {
			return
				(b.Min.X >= this.Min.X) && (b.Max.X <= this.Max.X) &&
				(b.Min.Y >= this.Min.Y) && (b.Max.Y <= this.Max.Y) &&
				(b.Min.Z >= this.Min.Z) && (b.Max.Z <= this.Max.Z);
		}

		/// <summary>Returns a formatted string describing the bounding box.</summary>
		public override string ToString() {
			return this.Min.ToString() + "->" + this.Max.ToString();
		}

		#region Conversions

		/// <summary>Converts a BoundingBoxShort into a BoundingBox.</summary>
		/// <param name="box">The bounding box to convert.</param>
		public static implicit operator BoundingBox(BoundingBoxShort box) {
			return new BoundingBox(box.Min, box.Max);
		}

		/// <summary>Converts a BoundingBoxShort into a BoundingBox.</summary>
		/// <param name="box">The bounding box to convert.</param>
		public static implicit operator BoundingBoxShort(BoundingBox box) {
			return new BoundingBoxShort((Vector3Short)box.Min, (Vector3Short)box.Max);
		}

		#endregion

	}
}
