using System;

namespace id.Quake {
	/// <summary>Describes what happened during a collision.</summary>
	public struct CollisionResult<T> {

		/// <summary>True if a collision occurred, false otherwise.</summary>
		public bool CollisionOccured;

		/// <summary>The position that was hit.</summary>
		public T Position;

		/// <summary>The face that was hit.</summary>
		public Files.Level.Face Face;

		/// <summary>The plane that was hit.</summary>
		public Plane? Plane;

		/// <summary>The BSP model that was hit.</summary>
		public Files.Level.Model Model;

		/// <summary>The distance along the path that the collision takes place.</summary>
		/// <remarks>0 corresponds to the start of the path, 1 corresponds to the end.</remarks>
		public float Distance;


		/// <summary>Create an instance of the collision class.</summary>
		/// <param name="collisionOccured">True if a collision has occured.</param>
		/// <param name="position">The position of the collision.</param>
		/// <param name="distance">The distance along the path that the collision takes place.</param>
		/// <param name="plane">The plane that is collided with.</param>
		public CollisionResult(bool collisionOccured, T position, float distance, Plane? plane) {
			this.CollisionOccured = collisionOccured;
			this.Position = position;
			this.Plane = plane;
			this.Model = null;
			this.Face = null;
			this.Distance = distance;
		}

		/// <summary>Create an instance of the collision class.</summary>
		/// <param name="collisionOccured">True if a collision has occured.</param>
		/// <param name="position">The position of the collision.</param>
		public CollisionResult(bool collisionOccured, T position)
			: this(collisionOccured, position, 1f, null) {
		}

	}
}
