using System;
using System.ComponentModel;

namespace id.Quake {

	/// <summary>Defines a vector with two components.</summary>
	public struct Vector2  {

		#region Properties

		/// <summary>Gets or sets the X component of the vector.</summary>
		public float X;

		/// <summary>Gets or sets the Y component of the vector.</summary>
		public float Y;

		#endregion

		#region Constructors

		/// <param name="value">Value to initialise each component to.</param>
		public Vector2(float value) {
			this.X = value; this.Y = value;
		}

		/// <param name="x">Initial value for the x-component of the vector.</param>
		/// <param name="y">Initial value for the y-component of the vector.</param>
		public Vector2(float x, float y) {
			this.X = x; this.Y = y;
		}

		/// <summary>Initialise a Vector3 reading its component values from a stream.</summary>
		/// <param name="stream"></param>
		public Vector2(System.IO.Stream stream) {
			System.IO.BinaryReader VectorReader = new System.IO.BinaryReader(stream);
			this.X = VectorReader.ReadSingle(); this.Y = VectorReader.ReadSingle();
		}

		#endregion

		#region Operators

		public static Vector2 operator +(Vector2 a, Vector2 b) { return new Vector2(a.X + b.X, a.Y + b.Y); }

		public static Vector2 operator +(Vector2 a, float b) { return a + new Vector2(b); }

		public static Vector2 operator +(float a, Vector2 b) { return new Vector2(a) + b; }

		public static Vector2 operator -(Vector2 a, Vector2 b) { return new Vector2(a.X - b.X, a.Y - b.Y); }

		public static Vector2 operator -(Vector2 a, float b) { return a - new Vector2(b); }

		public static Vector2 operator -(float a, Vector2 b) { return new Vector2(a) - b; }

		public static Vector2 operator *(Vector2 a, Vector2 b) { return new Vector2(a.X * b.X, a.Y * b.Y); }

		public static Vector2 operator *(Vector2 a, float b) { return a * new Vector2(b); }

		public static Vector2 operator *(float a, Vector2 b) { return new Vector2(a) * b; }

		public static Vector2 operator /(Vector2 a, Vector2 b) { return new Vector2(a.X / b.X, a.Y / b.Y); }

		public static Vector2 operator /(Vector2 a, float b) { return a / new Vector2(b); }

		public static Vector2 operator /(float a, Vector2 b) { return new Vector2(a) / b; }

		#endregion

		#region Methods

		public static Vector2 Min(Vector2 a, Vector2 b) {
			return new Vector2(Math.Min(a.X, b.X), Math.Min(a.Y, b.Y));
		}

		public static Vector2 Max(Vector2 a, Vector2 b) {
			return new Vector2(Math.Max(a.X, b.X), Math.Max(a.Y, b.Y));
		}

		#endregion

		public override string ToString() {
			return string.Format("{{{0}, {1}}}", this.X, this.Y);
		}



	}


}