using System;

namespace id.Quake {
	public partial class VirtualMachine {

		/// <summary>Runs a configuration file.</summary>
		/// <param name="config">The configuration file.</param>
		public void RunConfigurationFile(Files.Configuration config) {
			string[] Lines = config.Text.Split('\n');
			foreach (string Line in Lines) {
				this.ExecuteConsoleCommand(Line);
			}
		}

		/// <summary>Execute a console command.</summary>
		/// <param name="command">The command to execute.</param>
		public void ExecuteConsoleCommand(string command) {
			
			string Line = command.Split(new string[] { "//" }, StringSplitOptions.None)[0].Trim();
			
			if (string.IsNullOrEmpty(Line)) return;

			string Verb = Line.Split(' ', '\t')[0].Trim().ToLowerInvariant();
			string Arguments = Line.Substring(Verb.Length).Trim();

			ConsoleVariable CVar;
			if (this.ConsoleVariables.TryGetValue(Verb, out CVar)) {
				CVar.String = Arguments.Trim('"');
				return;
			}
		

			switch (Verb) {
				case "alias":
					break;
				case "exec":
					this.RunConfigurationFile(this.Resources.LoadObject<Files.Configuration>(Arguments));
					break;
				case "bind":
					break;
				case "unbindall":
					break;
				case "stuffcmds":
					break;
				case "startdemos":
					break;
				default:
					break;
			}
		}

	}
}
