using System;
using System.Collections.Generic;
using System.Text;
using Xna = Microsoft.Xna.Framework;
using Quake = id.Quake;
using IO = System.IO;

namespace XnaQuake {
	class Utility {

		public static Xna.Vector3 ConvertPosition(Quake.Vector3 input) {
			return new Xna.Vector3(-input.Y, input.Z, -input.X);
		}

		public static Quake.Vector3 ConvertPosition(Xna.Vector3 input) {
			return new Quake.Vector3(-input.Z, -input.X, input.Y);
		}

		public static Xna.Vector2 ConvertPosition(Quake.Vector2 input) {
			return new Xna.Vector2(input.X, input.Y);
		}

		public static Xna.BoundingBox ConvertPosition(Quake.BoundingBox input) {
			return new Xna.BoundingBox(Utility.ConvertPosition(input.Min), Utility.ConvertPosition(input.Max));
		}
		public static Quake.BoundingBox ConvertPosition(Xna.BoundingBox input) {
			return new Quake.BoundingBox(Utility.ConvertPosition(input.Min), Utility.ConvertPosition(input.Max));
		}

		public static Xna.Graphics.Texture2D TextureFromPicture(Xna.Graphics.GraphicsDevice graphics, Quake.Picture picture) {
			Xna.Graphics.Texture2D Texture = new Microsoft.Xna.Framework.Graphics.Texture2D(graphics, picture.Width, picture.Height, 0, Xna.Graphics.ResourceUsage.AutoGenerateMipMap, Xna.Graphics.SurfaceFormat.Color);
			Texture.SetData<int>(picture.Data);
			return Texture;
		}

		public static Xna.Graphics.Texture2D TextureFromMipMappedPicture(Xna.Graphics.GraphicsDevice graphics, id.Quake.Files.IMipMappedPicture picture) {
			Xna.Graphics.Texture2D Texture = null;
			for (int i = 0; i < 4; ++i) {
				id.Quake.Picture Picture = picture.GetPicture(i).GetNearestPowerOfTwo();
				if (Texture == null) {
					try {
						Texture = new Xna.Graphics.Texture2D(graphics, Picture.Width, Picture.Height, 4, Xna.Graphics.ResourceUsage.None, Xna.Graphics.SurfaceFormat.Color);
					} catch { return null; }
				}
				Texture.SetData<int>(i, null, Picture.Data, 0, Picture.Data.Length, Xna.Graphics.SetDataOptions.Discard);
			}
			return Texture;
		}

		public static Xna.Plane ConvertPlane(Quake.Files.Level.Plane plane) {
			return new Xna.Plane(ConvertPosition(plane.Normal), plane.Offset);
		}

		public static Xna.Graphics.Texture2D TextureFromStream(Xna.Graphics.GraphicsDevice graphics, IO.Stream stream) {
			Xna.Graphics.TextureCreationParameters CreationParameters = Xna.Graphics.Texture2D.GetCreationParameters(graphics, stream);
			CreationParameters.MipFilter = 0;
			CreationParameters.ResourceUsage = Xna.Graphics.ResourceUsage.AutoGenerateMipMap;
			stream.Seek(0, IO.SeekOrigin.Begin);
			return Xna.Graphics.Texture2D.FromFile(graphics, stream, CreationParameters);
		}

		public static float AngleQuakeToXna(float a) {
			return Xna.MathHelper.ToRadians(a);
		}

		public static float AngleXnaToQuake(float a) {
			return -Xna.MathHelper.ToDegrees(a);
		}

		public static Xna.Vector3 ConvertAngle(Quake.Vector3 b) {
			return new Xna.Vector3(AngleQuakeToXna(b.Y), AngleQuakeToXna(-b.X), AngleQuakeToXna(b.Z));
		}

		public static Quake.Vector3 ConvertAngle(Xna.Vector3 b) {
			return new Quake.Vector3(AngleXnaToQuake(b.Y), AngleXnaToQuake(-b.X), AngleXnaToQuake(b.Z));
		}
	}
}
